@@ -4,44 +4,44 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '1.3999_04';
+our $VERSION = '1.6000_01';
 
 use Carp              ();
-use Fcntl             ();
 use File::Spec        ();
 use Exporter          ();
 use Data::MessagePack ();
 use Scalar::Util      ();
 
-use Text::Xslate::Util qw(
-    mark_raw unmark_raw
-    html_escape escaped_string
-    uri_escape html_builder
-);
+use Text::Xslate::Util ();
+BEGIN {
+    # all the exportable functions are defined in ::Util
+    our @EXPORT_OK = qw(
+        mark_raw
+        unmark_raw
+        escaped_string
+        html_escape
+        uri_escape
+        html_builder
+    );
+    Text::Xslate::Util->import(@EXPORT_OK);
+}
 
 our @ISA = qw(Text::Xslate::Engine);
 
-our @EXPORT_OK = qw(
-    mark_raw
-    unmark_raw
-    escaped_string
-    html_escape
-    uri_escape
-    html_builder
-);
-
-my $BYTECODE_VERSION = '1.5';
+my $BYTECODE_VERSION = '1.6';
 
 # $bytecode_version + $fullpath + $compiler_and_parser_options
 my $XSLATE_MAGIC   = qq{xslate;$BYTECODE_VERSION;%s;%s;};
 
 # load backend (XS or PP)
+my $use_xs = 0;
 if(!exists $INC{'Text/Xslate/PP.pm'}) {
     my $pp = ($Text::Xslate::Util::DEBUG =~ /\b pp \b/xms or $ENV{PERL_ONLY});
     if(!$pp) {
         eval {
             require XSLoader;
             XSLoader::load(__PACKAGE__, $VERSION);
+            $use_xs = 1;
         };
         die $@ if $@ && $Text::Xslate::Util::DEBUG =~ /\b xs \b/xms; # force XS
     }
@@ -49,6 +49,7 @@ if(!exists $INC{'Text/Xslate/PP.pm'}) {
         require 'Text/Xslate/PP.pm';
     }
 }
+sub USE_XS() { $use_xs }
 
 # workaround warnings about numeric when it is a developpers' version
 # it must be here because the bootstrap routine requires the under bar.
@@ -57,7 +58,7 @@ $VERSION =~ s/_//;
 # for error messages (see T::X::Util)
 sub input_layer { ref($_[0]) ? $_[0]->{input_layer} : ':utf8' }
 
-package Text::Xslate::Engine;
+package Text::Xslate::Engine; # XS/PP common base class
 
 use Text::Xslate::Util qw(
     make_error
@@ -97,8 +98,9 @@ my %parser_option = (
 my %compiler_option = (
     syntax     => undef,
     type       => undef,
-    header     => undef,
-    footer     => undef,
+    header     => undef, # template augment
+    footer     => undef, # template agument
+    macro      => undef, # template augment
 );
 
 my %builtin = (
@@ -120,6 +122,20 @@ my %builtin = (
 
 sub default_functions { +{} } # overridable
 
+sub parser_option { # overridable
+    return \%parser_option;
+}
+
+sub compiler_option { # overridable
+    return \%compiler_option;
+}
+
+sub replace_option_value_for_magic_token { # overridable
+    #my($self, $name, $value) = @_;
+    #$value;
+    return $_[2];
+}
+
 sub options { # overridable
     my($self) = @_;
     return {
@@ -131,14 +147,15 @@ sub options { # overridable
         cache_dir    => _DEFAULT_CACHE_DIR,
         module       => undef,
         function     => undef,
+        html_builder_module => undef,
         compiler     => 'Text::Xslate::Compiler',
 
         verbose      => 1,
         warn_handler => undef,
         die_handler  => undef,
 
-        %parser_option,
-        %compiler_option,
+        %{ $self->parser_option },
+        %{ $self->compiler_option },
     };
 }
 
@@ -149,7 +166,7 @@ sub new {
     my $options = $class->options;
     my $used    = 0;
     my $nargs   = scalar keys %args;
-    while(my $key = each %{$options}) {
+    foreach my $key(keys %{$options}) {
         if(exists $args{$key}) {
             $used++;
         }
@@ -159,7 +176,7 @@ sub new {
     }
 
     if($used != $nargs) {
-        my @unknowns = grep { !exists $options->{$_} } keys %args;
+        my @unknowns = sort grep { !exists $options->{$_} } keys %args;
         warnings::warnif(misc
             => "$class: Unknown option(s): " . join ' ', @unknowns);
     }
@@ -169,41 +186,52 @@ sub new {
             ref($args{path}) eq 'ARRAY' ? @{$args{path}} : $args{path}
     ];
 
-    # function
-    my %added_funcs;
-    $class->_merge_hash(\%added_funcs, $class->default_functions());
+    my $arg_function= $args{function};
 
-    # 'module' overrides default functions
+    my %funcs;
+    $args{function} = \%funcs;
+
+    $args{template} = {}; # template structures
+
+    my $self = bless \%args, $class;
+
+    # definition of functions and methods
+
+    # builtin functions
+    %funcs = %builtin;
+    $self->_register_builtin_methods(\%funcs);
+
+    # per-class functions
+    $self->_merge_hash(\%funcs, $class->default_functions());
+
+    # user-defined functions
     if(defined $args{module}) {
-        $class->_merge_hash(\%added_funcs,
+        $self->_merge_hash(\%funcs,
             Text::Xslate::Util::import_from(@{$args{module}}));
     }
 
-    # 'function' overrides imported functons
-    $class->_merge_hash(\%added_funcs, $args{function});
+    # user-defined html builder functions
+    if(defined $args{html_builder_module}) {
+        my $raw = Text::Xslate::Util::import_from(@{$args{html_builder_module}});
+        my $html_builders = +{
+            map {
+                ($_ => &Text::Xslate::Util::html_builder($raw->{$_}))
+            } keys %$raw
+        };
+        $self->_merge_hash(\%funcs, $html_builders);
+    }
 
-    my %funcs = %builtin;
-    $class->_register_builtin_methods(\%funcs);
-    # user defined functions (added functions) can override builtins
-    $class->_merge_hash(\%funcs, \%added_funcs);
+    $self->_merge_hash(\%funcs, $arg_function);
 
-    $class->_resolve_function_aliases(\%funcs);
+    $self->_resolve_function_aliases(\%funcs);
 
-    $args{function} = \%funcs;
-
-    # used for the magic token
-    $args{added_function_names} = [sort keys %added_funcs];
-
-    # internal data
-    $args{template} = {};
-
-    return bless \%args, $class;
+    return $self;
 }
 
 sub _merge_hash {
     my($self, $base, $add) = @_;
-    while(my($name, $body) = each %{$add}) {
-        $base->{$name} = $body;
+    foreach my $name(keys %{$add}) {
+        $base->{$name} = $add->{$name};
     }
     return;
 }
@@ -262,7 +290,7 @@ sub find_file {
     my $orig_mtime;
     my $cache_mtime;
     foreach my $p(@{$self->{path}}) {
-        $self->note("  find_file: %s / %s ...\n", $p, $file) if _DUMP_LOAD;
+        $self->note("  find_file: %s in  %s ...\n", $file, $p) if _DUMP_LOAD;
 
         my $cache_prefix;
         if(ref $p eq 'HASH') { # virtual path
@@ -281,6 +309,10 @@ sub find_file {
             defined($orig_mtime = (stat($fullpath))[_ST_MTIME])
                 or next;
             $cache_prefix = Text::Xslate::uri_escape($p);
+            if (length $cache_prefix > 127) {
+                # some filesystems refuse a path part with length > 127
+                $cache_prefix = $self->_digest($cache_prefix);
+            }
         }
 
         # $file is found
@@ -298,7 +330,7 @@ sub find_file {
         $self->_error("LoadError: Cannot find '$file' (path: @{$self->{path}})");
     }
 
-    $self->note("  find_file: %s (%d)\n",
+    $self->note("  find_file: %s (mtime=%d)\n",
         $fullpath, $cache_mtime || 0) if _DUMP_LOAD;
 
     return {
@@ -313,11 +345,11 @@ sub find_file {
 
 
 sub load_file {
-    my($self, $file, $mtime, $from_include) = @_;
+    my($self, $file, $mtime, $omit_augment) = @_;
 
-    local $self->{from_include} = $from_include;
+    local $self->{omit_augment} = $omit_augment;
 
-    $self->note("load_file(%s)\n", $file) if _DUMP_LOAD;
+    $self->note("%s->load_file(%s)\n", $self, $file) if _DUMP_LOAD;
 
     if($file eq '<string>') { # simply reload it
         return $self->load_string($self->{string_buffer});
@@ -339,12 +371,11 @@ sub load_file {
     return $asm;
 }
 
-sub slurp {
-    my($self, $fullpath) = @_;
+sub slurp_template {
+    my($self, $input_layer, $fullpath) = @_;
 
-    open my($source), '<' . $self->{input_layer}, $fullpath
+    open my($source), '<' . $input_layer, $fullpath
         or $self->_error("LoadError: Cannot open $fullpath for reading: $!");
-    flock($source, Fcntl::LOCK_SH());
     local $/;
     return scalar <$source>;
 }
@@ -363,7 +394,7 @@ sub _load_source {
             or Carp::carp("Xslate: cannot unlink $cachepath (ignored): $!");
     }
 
-    my $source = $self->slurp($fullpath);
+    my $source = $self->slurp_template($self->input_layer, $fullpath);
     $self->{source}{$fi->{name}} = $source if _SAVE_SRC;
 
     my $asm = $self->compile($source,
@@ -377,21 +408,35 @@ sub _load_source {
         if(not -e $cachedir) {
             require File::Path;
             eval { File::Path::mkpath($cachedir) }
-                or Carp::carp("Xslate: Cannot make directory $cachepath (ignored): $@");
+                or Carp::croak("Xslate: Cannot prepare cache directory $cachepath (ignored): $@");
         }
 
-        if(sysopen my($out), $cachepath, Fcntl::O_WRONLY() | Fcntl::O_CREAT()) {
-            flock $out, Fcntl::LOCK_EX();
-            truncate $out, 0 or Carp::croak("Xslate: failed to truncate: $!");
-            binmode($out);
-            $self->_save_compiled($out, $asm, $fullpath, utf8::is_utf8($source));
+        my $tmpfile = sprintf('%s.%d.d', $cachepath, $$, $self);
+
+        if (open my($out), ">:raw", $tmpfile) {
+            my $mtime = $self->_save_compiled($out, $asm, $fullpath, utf8::is_utf8($source));
 
             if(!close $out) {
                  Carp::carp("Xslate: Cannot close $cachepath (ignored): $!");
-                 unlink $cachepath;
+                 unlink $tmpfile;
+            }
+            elsif (rename($tmpfile => $cachepath)) {
+                # set the newest mtime of all the related files to cache mtime
+                if (not ref $fullpath) {
+                    my $main_mtime = (stat $fullpath)[_ST_MTIME];
+                    if (defined($main_mtime) && $main_mtime > $mtime) {
+                        $mtime = $main_mtime;
+                    }
+                    utime $mtime, $mtime, $cachepath;
+                    $fi->{cache_mtime} = $mtime;
+                }
+                else {
+                    $fi->{cache_mtime} = (stat $cachepath)[_ST_MTIME];
+                }
             }
             else {
-                $fi->{cache_mtime} = ( stat $cachepath )[_ST_MTIME];
+                Carp::carp("Xslate: Cannot rename cache file $cachepath (ignored): $!");
+                unlink $tmpfile;
             }
         }
         else {
@@ -399,7 +444,7 @@ sub _load_source {
         }
     }
     if(_DUMP_LOAD) {
-        $self->note("  _load_source: cache(%s)\n",
+        $self->note("  _load_source: cache(mtime=%s)\n",
             defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef');
     }
 
@@ -430,7 +475,6 @@ sub _load_compiled {
     my $cachepath = $fi->{cachepath};
     open my($in), '<:raw', $cachepath
         or $self->_error("LoadError: Cannot open $cachepath for reading: $!");
-    flock($in, Fcntl::LOCK_SH());
 
     my $magic = $self->_magic_token($fi->{fullpath});
     my $data;
@@ -451,6 +495,11 @@ sub _load_compiled {
     $unpacker->utf8($is_utf8);
 
     my @asm;
+    if($is_utf8) { # TODO: move to XS?
+        my $seed = "";
+        utf8::upgrade($seed);
+        push @asm, ['print_raw_s', $seed, __LINE__, __FILE__];
+    }
     while($offset < length($data)) {
         $offset = $unpacker->execute($data, $offset);
         my $c = $unpacker->data();
@@ -475,7 +524,7 @@ sub _load_compiled {
     }
 
     if(_DUMP_LOAD) {
-        $self->note("  _load_compiled: cache(%s)\n",
+        $self->note("  _load_compiled: cache(mtime=%s)\n",
             defined $fi->{cache_mtime} ? $fi->{cache_mtime} : 'undef');
     }
 
@@ -484,13 +533,23 @@ sub _load_compiled {
 
 sub _save_compiled {
     my($self, $out, $asm, $fullpath, $is_utf8) = @_;
+    my $mp = Data::MessagePack->new();
     local $\;
     print $out $self->_magic_token($fullpath);
-    print $out Data::MessagePack->pack($is_utf8 ? 1 : 0);
+    print $out $mp->pack($is_utf8 ? 1 : 0);
+
+    my $newest_mtime = 0;
     foreach my $c(@{$asm}) {
-        print $out Data::MessagePack->pack($c);
+        print $out $mp->pack($c);
+
+        if ($c->[0] eq 'depend') {
+            my $dep_mtime = (stat $c->[1])[_ST_MTIME];
+            if ($newest_mtime < $dep_mtime) {
+                $newest_mtime = $dep_mtime;
+            }
+        }
     }
-    return;
+    return $newest_mtime;
 }
 
 sub _magic_token {
@@ -498,9 +557,10 @@ sub _magic_token {
 
     $self->{serial_opt} ||= Data::MessagePack->pack([
         ref($self->{compiler}) || $self->{compiler},
-        $self->_extract_options(\%parser_option),
-        $self->_extract_options(\%compiler_option),
-        $self->{added_function_names},
+        $self->_filter_options_for_magic_token($self->_extract_options($self->parser_option)),
+        $self->_filter_options_for_magic_token($self->_extract_options($self->compiler_option)),
+        $self->input_layer,
+        [sort keys %{ $self->{function} }],
     ]);
 
     if(ref $fullpath) { # ref to content string
@@ -530,6 +590,19 @@ sub _extract_options {
     return @options;
 }
 
+sub _filter_options_for_magic_token {
+    my($self, @options) = @_;
+    my @filterd_options;
+    while (@options) {
+        my $name  = shift @options;
+        my $value = $self->replace_option_value_for_magic_token($name, shift @options);
+        push(@filterd_options, $name => $value);
+    }
+    @filterd_options;
+}
+
+
+
 sub _compiler {
     my($self) = @_;
     my $compiler = $self->{compiler};
@@ -542,10 +615,10 @@ sub _compiler {
         $compiler = $compiler->new(
             engine      => $self,
             input_layer => $input_layer,
-            $self->_extract_options(\%compiler_option),
+            $self->_extract_options($self->compiler_option),
             parser_option => {
                 input_layer => $input_layer,
-                $self->_extract_options(\%parser_option),
+                $self->_extract_options($self->parser_option),
             },
         );
 
@@ -560,7 +633,7 @@ sub _compiler {
 sub compile {
     my $self = shift;
     return $self->_compiler->compile(@_,
-        from_include => $self->{from_include});
+        omit_augment => $self->{omit_augment});
 }
 
 sub _error {
@@ -582,7 +655,7 @@ Text::Xslate - Scalable template engine for Perl5
 
 =head1 VERSION
 
-This document describes Text::Xslate version 1.3999_04.
+This document describes Text::Xslate version 1.6000_01.
 
 =head1 SYNOPSIS
 
@@ -660,11 +733,11 @@ L<http://illusori.co.uk/projects/Template-Roundup/201010/performance_vs_variant_
 
 There are some benchmarks in F<benchmark/> directory in the Xslate distribution.
 
-=head3 Smart escaping for HTML meta characters
+=head3 Smart escaping for HTML metacharacters
 
-All the HTML meta characters in template expressions the engine interpolates
-into template texts are escaped automatically, so the output has no
-possibility to XSS by default.
+All HTML metacharacters in template expressions which are interpolated into
+template texts by the engine are escaped automatically. This means that, by default,
+the output is not prone to XSS.
 
 =head3 Template cascading
 
@@ -722,7 +795,31 @@ template names.
 Specifies a function map which contains name-coderef pairs.
 A function C<f> may be called as C<f($arg)> or C<$arg | f> in templates.
 
-There are built-in functions described in L<Text::Xslate::Manual::Builtin>.
+Note that those registered function have to return a B<text string>,
+not a binary string unless you want to handle bytes in whole templates.
+Make sure what you want to use returns whether text string or binary
+strings.
+
+For example, some methods of C<Time::Piece> might return a binary string
+which is encoded in UTF-8, so you'd like to decode their values.
+
+    # under LANG=ja_JP.UTF-8 on MacOSX (Darwin 11.2.0)
+    use Time::Piece;
+    use Encode qw(decode);
+
+    sub ctime {
+        my $ctime = Time::Piece->new->strftime; # UTF-8 encoded bytes
+        return decode "UTF-8", $ctime;
+    }
+
+    my $tx = Text::Xslate->new(
+        function => {
+            ctime => \&ctime,
+        },
+        ...,
+    );
+
+Built-in functions are described in L<Text::Xslate::Manual::Builtin>.
 
 =item C<< module => [$module => ?\@import_args, ...] >>
 
@@ -733,26 +830,30 @@ For example:
 
     # for function-based modules
     my $tx = Text::Xslate->new(
-        module => ['Time::Piece'],
+        module => ['Digest::SHA1' => [qw(sha1_hex)]],
     );
     print $tx->render_string(
-        '<: localtime($x).strftime() :>',
-        { x => time() },
-    ); # => Wed, 09 Jun 2010 10:22:06 JST
+        '<: sha1_hex($x).substr(0, 6) :>',
+        { x => foo() },
+    ); # => 0beec7
 
     # for bridge modules
     my $tx = Text::Xslate->new(
-        module => ['SomeModule::Bridge::Xslate'],
+        module => ['Text::Xslate::Bridge::Star'],
     );
     print $tx->render_string(
-        '<: $x.some_method() :>',
-        { x => time() },
-    );
+        '<: $x.uc() :>',
+        { x => 'foo' },
+    ); # => 'FOO'
 
 Because you can use function-based modules with the C<module> option, and
 also can invoke any object methods in templates, Xslate doesn't require
 specific namespaces for plugins.
 
+=item C<< html_builder_module => [$module => ?\@import_args, ...] >>
+
+Imports functions from I<$module>, wrapping each function with C<html_builder()>.
+
 =item C<< input_layer => $perliolayers // ':utf8' >>
 
 Specifies PerlIO layers to open template files.
@@ -831,14 +932,10 @@ This option is passed to the compiler.
 
 Specify the callback I<&cb> which is called on warnings.
 
-This option is experimental.
-
 =item C<< die_handler => \&cb >>
 
 Specify the callback I<&cb> which is called on fatal errors.
 
-This option is experimental.
-
 =back
 
 =head3 B<< $tx->render($file, \%vars) :Str >>
@@ -876,6 +973,11 @@ Compiles and saves it as disk caches if needed.
 Returns the current Xslate engine while executing. Otherwise returns C<undef>.
 This method is significant when it is called by template functions and methods.
 
+=head3 B<< Text::Xslate->current_vars :HashRef >>
+
+Returns the current variable table, namely the second argument of
+C<render()> while executing. Otherwise returns C<undef>.
+
 =head3 B<< Text::Xslate->current_file :Str >>
 
 Returns the current file name while executing. Otherwise returns C<undef>.
@@ -1080,7 +1182,7 @@ An "too-safe" HTML escaping filter which escape all the symbolic characters
 
 =cut
 
-=head1 SUPPORT
+=head1 RESOURCES
 
 WEB: L<http://xslate.org/>
 
@@ -1088,9 +1190,9 @@ ML: L<http://groups.google.com/group/xslate>
 
 IRC: #xslate @ irc.perl.org
 
-REPOSITORY:
-    http://github.com/gfx/p5-Text-Xslate/
-    git://github.com/gfx/p5-Text-Xslate.git
+PROJECT HOME: L<http://github.com/xslate/>
+
+REPOSITORY: L<http://github.com/xslate/p5-Text-Xslate/>
 
 =head1 BUGS
 
@@ -1184,6 +1286,8 @@ Thanks to cho45 for the bug reports.
 
 Thanks to shmorimo for the bug reports.
 
+Thanks to ueda for the suggestions.
+
 =head1 AUTHOR
 
 Fuji, Goro (gfx) E<lt>gfuji(at)cpan.orgE<gt>
