@@ -3,7 +3,7 @@ package Text::Xslate::PP;
 use 5.008_001;
 use strict;
 
-our $VERSION = '1.3999_04';
+our $VERSION = '1.6000_01';
 
 BEGIN{
     $ENV{XSLATE} = ($ENV{XSLATE} || '') . '[pp]';
@@ -117,6 +117,10 @@ sub current_engine {
     return defined($_current_st) ? $_current_st->engine : undef;
 }
 
+sub current_vars {
+    return defined($_current_st) ? $_current_st->vars : undef;
+}
+
 sub current_file {
     return defined($_current_st)
         ? $_current_st->code->[ $_current_st->{ pc } ]->{file}
@@ -343,6 +347,9 @@ sub _assemble {
 
     sub is_array_ref { ref($_[0]) eq 'ARRAY' }
     sub is_hash_ref  { ref($_[0]) eq 'HASH'  }
+    sub is_code_ref  { ref($_[0]) eq 'CODE'  }
+
+    sub merge_hash { +{ %{ $_[0] }, %{ $_[1] } } }
 }
 
 #
@@ -639,7 +646,7 @@ Text::Xslate::PP - Yet another Text::Xslate runtime in pure Perl
 
 =head1 VERSION
 
-This document describes Text::Xslate::PP version 1.3999_04.
+This document describes Text::Xslate::PP version 1.6000_01.
 
 =head1 DESCRIPTION
 
